import org.jblas.DoubleMatrix;
import org.jblas.FloatMatrix;

import org.shogun.shogun;
import org.shogun.AccuracyMeasure;
import org.shogun.BinaryLabels;
import org.shogun.CSVFile;
import org.shogun.CombinedFeatures;
import org.shogun.CombinedKernel;
import org.shogun.CrossValidation;
import org.shogun.CrossValidationFoldStorage;
import org.shogun.CrossValidationResult;
import org.shogun.CrossValidationStorage;
import org.shogun.GaussianKernel;
import org.shogun.LibSVM;
import org.shogun.MKLClassification;
import org.shogun.ParameterObserverCV;
import org.shogun.RealFeatures;
import org.shogun.RealVector;
import org.shogun.StratifiedCrossValidationSplitting;

public class cross_validation_mkl_weight_storage {
static {
System.loadLibrary("shogun");
}

public static void main(String argv[]) {
shogun.init_shogun_with_defaults();

CSVFile f_feats = new CSVFile("../../data/fm_train_real.dat");
CSVFile f_labels = new CSVFile("../../data/label_train_twoclass.dat");

//![create_features]
RealFeatures features = new RealFeatures(f_feats);
CombinedFeatures combined_features = new CombinedFeatures();
combined_features.append_feature_obj(features);
combined_features.append_feature_obj(features);
combined_features.append_feature_obj(features);
BinaryLabels labels = new BinaryLabels(f_labels);
//![create_features]

//![create_kernels]
CombinedKernel kernel = new CombinedKernel();
GaussianKernel k_1 = new GaussianKernel(10, 0.1);
GaussianKernel k_2 = new GaussianKernel(10, 1);
GaussianKernel k_3 = new GaussianKernel(10, 2);
kernel.append_kernel(k_1);
kernel.append_kernel(k_2);
kernel.append_kernel(k_3);
//![create_kernels]

//![create_classifier]
LibSVM libsvm = new LibSVM();
MKLClassification svm = new MKLClassification(libsvm);
svm.set_interleaved_optimization_enabled(false);
svm.set_kernel(kernel);
//![create_classifier]

//![create_cross_validation]
StratifiedCrossValidationSplitting splitting_strategy = new StratifiedCrossValidationSplitting(labels, 5);
AccuracyMeasure evaluation_criterium = new AccuracyMeasure();
CrossValidation cross = new CrossValidation(svm, combined_features, labels, splitting_strategy, evaluation_criterium);
cross.set_autolock(false);
cross.set_num_runs(2);
//![create_cross_validation]

//![create_observer]
ParameterObserverCV mkl_obs = new ParameterObserverCV(true);
cross.subscribe_to_parameters(mkl_obs);
//[!create_observer]

//![evaluate_and_get_result]
CrossValidationResult result = new CrossValidationResult();
result = CrossValidationResult.obtain_from_generic(cross.evaluate());
//![evaluate_and_get_result]

//![get_results]
double mean = result.get_mean();
double stddev = result.get_std_dev();
//![get_results]

//![get_fold_machine]
CrossValidationStorage obs = mkl_obs.get_observation(0);
CrossValidationFoldStorage fold = obs.get_fold(0);
MKLClassification machine = MKLClassification.obtain_from_generic(fold.get_trained_machine());
//![get_fold_machine]

//![get_weights]
CombinedKernel k = CombinedKernel.obtain_from_generic(machine.get_kernel());
DoubleMatrix w = k.get_subkernel_weights();
//![get_weights]

}
}
