import org.jblas.DoubleMatrix;
import org.jblas.FloatMatrix;

import org.shogun.shogun;
import org.shogun.CSVFile;
import org.shogun.ExactInferenceMethod;
import org.shogun.GaussianKernel;
import org.shogun.GaussianLikelihood;
import org.shogun.GaussianProcessRegression;
import org.shogun.GradientCriterion;
import org.shogun.GradientEvaluation;
import org.shogun.GradientModelSelection;
import org.shogun.MeanSquaredError;
import org.shogun.ParameterCombination;
import org.shogun.RealFeatures;
import org.shogun.RealVector;
import org.shogun.RegressionLabels;
import org.shogun.ZeroMean;

public class gaussian_process_regression {
static {
System.loadLibrary("shogun");
}

public static void main(String argv[]) {
shogun.init_shogun_with_defaults();

CSVFile f_feats_train = new CSVFile("../../data/regression_1d_sinc_features_train.dat");
CSVFile f_feats_test = new CSVFile("../../data/regression_1d_sinc_features_test.dat");
CSVFile f_labels_train = new CSVFile("../../data/regression_1d_sinc_labels_train.dat");
CSVFile f_labels_test = new CSVFile("../../data/regression_1d_sinc_labels_test.dat");

//![create_features]
RealFeatures features_train = new RealFeatures(f_feats_train);
RealFeatures features_test = new RealFeatures(f_feats_test);
RegressionLabels labels_train = new RegressionLabels(f_labels_train);
RegressionLabels labels_test = new RegressionLabels(f_labels_test);
//![create_features]

//![create_appropriate_kernel_and_mean_function]
double width = 1.0;
GaussianKernel kernel = new GaussianKernel(features_train, features_train, width);
ZeroMean mean_function = new ZeroMean();
//![create_appropriate_kernel_and_mean_function]

//![create_instance]
GaussianLikelihood gauss_likelihood = new GaussianLikelihood();
ExactInferenceMethod inference_method = new ExactInferenceMethod(kernel, features_train, mean_function, labels_train, gauss_likelihood);
GaussianProcessRegression gp_regression = new GaussianProcessRegression(inference_method);
//![create_instance]

//![train_and_apply]
gp_regression.train();
RegressionLabels labels_predict = gp_regression.apply_regression(features_test);
//![train_and_apply]

//![compute_variance]
DoubleMatrix variances = gp_regression.get_variance_vector(features_test);
//![compute_variance]

//![optimize_marginal_likelihood]
GradientCriterion grad_criterion = new GradientCriterion();
GradientEvaluation grad = new GradientEvaluation(gp_regression, features_train, labels_train, grad_criterion);
grad.set_function(inference_method);
GradientModelSelection grad_selection = new GradientModelSelection(grad);
ParameterCombination best_theta = grad_selection.select_model();
best_theta.apply_to_machine(gp_regression);
//![optimize_marginal_likelihood]

//![evaluate_error_and_marginal_likelihood]
MeanSquaredError eval = new MeanSquaredError();
double mse = eval.evaluate(labels_predict, labels_test);
double marg_ll = inference_method.get_negative_log_marginal_likelihood();
//![evaluate_error_and_marginal_likelihood]

// integration testing variables
DoubleMatrix output = labels_test.get_labels();

}
}
